/*******************************************************************************

 WINBINDER - A native Windows binding for PHP

 Copyright  2004 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 ZEND wrapper for bitmap functions

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "phpwb.h"

//----------------------------------------------------------- EXPORTED FUNCTIONS

/*
 index is the index of the image on the file if filename is an icon library. Default is 0.
*/

ZEND_FUNCTION(wb_load_image)
{
	char *s;
	int s_len;
	long index;
	HANDLE hImage;

	// Default parameter values

	index = 0;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "s|l", &s, &s_len, &index) == FAILURE)
		return;

	hImage = wbLoadImage(s, index);
	if(!hImage) {
		RETURN_NULL();
	} else
		RETURN_LONG((long)hImage);
}

ZEND_FUNCTION(wb_save_image)
{
	long hbm;
	char *s;
	int s_len;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "ls", &hbm, &s, &s_len) == FAILURE)
		RETURN_BOOL(FALSE);

	if(!hbm)
		RETURN_NULL();

	RETURN_BOOL(wbSaveBitmap((HBITMAP)hbm, s));
}

ZEND_FUNCTION(wb_get_image_size)
{
    long hbm;
    DWORD dim;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "l", &hbm) == FAILURE)
		RETURN_BOOL(FALSE);

	if(!hbm)
		RETURN_NULL();

	dim = wbGetBitmapDimensions((HBITMAP)hbm);

	// Build the return array

	array_init(return_value);
	add_next_index_long(return_value, LOWORD(dim));
	add_next_index_long(return_value, HIWORD(dim));
}

ZEND_FUNCTION(wb_create_image)
{
    long w, h;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "ll", &w, &h) == FAILURE)
		return;

	RETURN_LONG((LONG)wbCreateBitmap(w, h))
}

// TODO: perform a non-destructive masking

ZEND_FUNCTION(wb_create_mask)
{
    long hbm, c;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "ll", &hbm, &c) == FAILURE)
		return;

	if(!hbm)
		RETURN_NULL();

	RETURN_LONG((LONG)wbCreateMask((HBITMAP)hbm, c))
}

ZEND_FUNCTION(wb_destroy_image)
{
    long hbm;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "l", &hbm) == FAILURE)
		return;

	if(!hbm)
		RETURN_NULL();

	RETURN_BOOL(wbDestroyBitmap((HBITMAP)hbm))
}

//------------------------------------------------------------------ END OF FILE
