/*******************************************************************************

 WINBINDER - A native Windows binding for PHP

 Copyright  2004 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 ZEND wrapper for window controls

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "phpwb.h"

//----------------------------------------------------------- EXPORTED FUNCTIONS

ZEND_FUNCTION(wbtemp_create_control)
{
    long wbclass, x, y, w, h, id, style, lparam;
	char *caption = "";
	int caption_len;
	long pwboparent;

	x = WBC_CENTER;
	y = WBC_CENTER;
	w = CW_USEDEFAULT;
	h = CW_USEDEFAULT;
	style = 0;
	lparam = 0;
	id = 32767;

	if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "ll|slllllll", &pwboparent, &wbclass, &caption, &caption_len, &x, &y, &w, &h, &id, &style, &lparam) == FAILURE)
		return;

	if(ZEND_NUM_ARGS() == 5) {
		w = x;
		h = y;
		x = WBC_CENTER;
		y = WBC_CENTER;
	}

	if(!wbIsWBObj((void *)pwboparent))
		RETURN_NULL()
	else
		RETURN_LONG((LONG)wbCreateControl((PWBOBJ)pwboparent, wbclass, caption, x, y, w, h, id, style, lparam));
}

ZEND_FUNCTION(wb_get_visible)
{
	long pwbo;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l", &pwbo) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()
	else
		RETURN_BOOL(wbGetVisible((PWBOBJ)pwbo));
}

ZEND_FUNCTION(wb_set_visible)
{
	long pwbo;
    long b;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "ll", &pwbo, &b) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()
	else
		RETURN_BOOL(wbSetVisible((PWBOBJ)pwbo, b));
}

ZEND_FUNCTION(wb_set_focus)
{
	long pwbo;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l", &pwbo) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()
	else
		RETURN_BOOL(wbSetFocus((PWBOBJ)pwbo))
}

ZEND_FUNCTION(wb_set_style)
{
	long pwbo, style;
	int type = 0;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "ll|l", &pwbo, &style, &type) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL();

//	if(!type) {
//		RETURN_BOOL(wbSetStyle((PWBOBJ)pwbo, style));
//	} else
		RETURN_BOOL(wbSetWindowStyle((PWBOBJ)pwbo, style, type > 1));
}

ZEND_FUNCTION(wb_get_class)
{
	long pwbo;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l", &pwbo) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL();

	RETURN_LONG(((PWBOBJ)pwbo)->uClass);
}

ZEND_FUNCTION(wb_get_value)
{
	long pwbo, item = 0, subitem = 0;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l|ll", &pwbo, &item, &subitem) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL();

	((PWBOBJ)pwbo)->item = item;
	((PWBOBJ)pwbo)->subitem = subitem;

	if(((PWBOBJ)pwbo)->uClass == ListView) {

		UINT i;
		char nitems;
		int *items;

		// How many items are checked?

		nitems = wbGetListViewCheckedItems((PWBOBJ)pwbo, NULL);
		if(!nitems)				// No items checked
			RETURN_NULL();

		// Call the function again, this time to fill up the item array

		items = emalloc(sizeof(int) * nitems);
		wbGetListViewCheckedItems((PWBOBJ)pwbo, items);

		// Build the return array

		array_init(return_value);
		for(i = 0; i < nitems; i++)
			add_next_index_long(return_value, items[i]);

		efree(items);

	} else
		RETURN_LONG(wbGetValue((PWBOBJ)pwbo));
}

ZEND_FUNCTION(wb_get_selected)
{
	long pwbo, item = 0, subitem = 0;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l|ll", &pwbo, &item, &subitem) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL();

	((PWBOBJ)pwbo)->item = item;
	((PWBOBJ)pwbo)->subitem = subitem;

	if(((PWBOBJ)pwbo)->uClass == TabControl) {

		RETURN_LONG(TabCtrl_GetCurSel(((PWBOBJ)pwbo)->handle));

	} else if(((PWBOBJ)pwbo)->uClass == ListView) {

		UINT i;
		char nitems;
		int *items;

		// How many items are selected?

		nitems = wbGetListViewSelectedItems((PWBOBJ)pwbo, NULL);
		if(!nitems)				// No items selected
			RETURN_NULL();

		// Call the function again, this time to fill up the item array

		items = emalloc(sizeof(int) * nitems);
		wbGetListViewSelectedItems((PWBOBJ)pwbo, items);

		// Build the return array

		array_init(return_value);
		for(i = 0; i < nitems; i++)
			add_next_index_long(return_value, items[i]);

		efree(items);

	} else
		RETURN_LONG(wbGetSelected((PWBOBJ)pwbo));
}

ZEND_FUNCTION(wb_set_image)
{
    long index, color, lparam;
	long pwbo;
    zval *source;
	HANDLE hImage;

	// Default parameter values

	index = 0;
	lparam = 0;
	color = CLR_INVALID;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l!z!|lll", &pwbo, &source, &index, &color, &lparam) == FAILURE)
		return;

	if(!source)
		RETURN_NULL();

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL();

	if(source->type == IS_LONG) {
		hImage = (HANDLE)source->value.lval;
	} else if(source->type == IS_STRING) {
		hImage = wbLoadImage(source->value.str.val, index);
//		printf("%s %d %d (%d) %d\n", source->value.str.val,
//			hImage, pwbo, wbIsWBObj((void *)pwbo), ((PWBOBJ)pwbo)->uClass);
	} else {
		zend_error(E_WARNING, "Invalid parameter type passed to function %s", __func__);
		RETURN_NULL();
	}

	RETURN_BOOL(wbSetImage((PWBOBJ)pwbo, hImage, index, color, lparam));
}

ZEND_FUNCTION(wb_get_control)
{
    long id;
	long pwboparent;

	id = 0;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l|l", &pwboparent, &id) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwboparent))
		RETURN_NULL()
	else
		RETURN_LONG((LONG)wbGetControl((PWBOBJ)pwboparent, id));
}

ZEND_FUNCTION(wb_set_enabled)
{
	long pwbo, state;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "ll", &pwbo, &state) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()
	else
		RETURN_BOOL(wbSetEnabled((PWBOBJ)pwbo, state));
}

ZEND_FUNCTION(wb_get_enabled)
{
	long pwbo;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l", &pwbo) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()
	else
		RETURN_BOOL(wbGetEnabled((PWBOBJ)pwbo));
}

/* bool wb_refresh (int control [, bool now]) */

ZEND_FUNCTION(wb_refresh)
{
	long pwbo, now = FALSE;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l|l", &pwbo, &now) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()
	else
		RETURN_BOOL(wbRefreshControl((PWBOBJ)pwbo, now));
}

ZEND_FUNCTION(wb_get_item_count)
{
	long pwbo;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l", &pwbo) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()
	else
		RETURN_LONG(wbGetItemCount((PWBOBJ)pwbo));
}

ZEND_FUNCTION(wb_delete_items)
{
	long pwbo, nstartitem = -1, nitems = 1;
	int i;
	BOOL success = TRUE;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "l|ll", &pwbo, &nstartitem, &nitems) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()

	if(nstartitem < 0) {							// Delete all items
		RETURN_BOOL(wbClearItems((PWBOBJ)pwbo));
	} else {
		nitems = min(1, nitems);
		if(((PWBOBJ)pwbo)->uClass == ListView) {	// Delete one or a range of items
			for(i = nstartitem; i < nstartitem + nitems; i++) {
				if(!wbDeleteListViewItem((PWBOBJ)pwbo, nstartitem))
					success = FALSE;
			}
			RETURN_BOOL(success);
		} else
			RETURN_FALSE;
	}
}

ZEND_FUNCTION(wb_sort)
{
	long pwbo, ascending = TRUE, subitem = 0;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l|ll", &pwbo, &ascending, &subitem) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()
	else {
		if(((PWBOBJ)pwbo)->uClass == ListView)
			RETURN_BOOL(wbSortLVColumn((PWBOBJ)pwbo, subitem, ascending))
		else
			RETURN_BOOL(FALSE);
	}
}

//------------------------------------------------- AUXILIARY EXPORTED FUNCTIONS

ZEND_FUNCTION(wbtemp_select_tab)
{
	long pwbo;
	int ntab;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "ll", &pwbo, &ntab) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL();

	RETURN_BOOL(wbSelectTab((PWBOBJ)pwbo, ntab));
}

ZEND_FUNCTION(wbtemp_set_value)
{
	long pwbo, value, item = 0, subitem = 0;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "ll|ll", &pwbo, &value, &item, &subitem) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL();

	((PWBOBJ)pwbo)->item = item;
	((PWBOBJ)pwbo)->subitem = subitem;

	RETURN_BOOL(wbSetValue((PWBOBJ)pwbo, value));
}

ZEND_FUNCTION(wbtemp_set_range)
{
	long pwbo, min = 0, max = 0;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "ll|l", &pwbo, &min, &max) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL();

	RETURN_BOOL(wbSetRange((PWBOBJ)pwbo, min, max));
}

ZEND_FUNCTION(wbtemp_create_item)
{
	char *s;
	int s_len;
	long pwbo;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "ls", &pwbo, &s, &s_len) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()
	else
		RETURN_BOOL(wbCreateItem((PWBOBJ)pwbo, s));
}

ZEND_FUNCTION(wbtemp_set_text)
{
	char *s;
	int s_len;
	long pwbo;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "ls", &pwbo, &s, &s_len) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()
	else
		RETURN_BOOL(wbSetText((PWBOBJ)pwbo, s));
}

ZEND_FUNCTION(wbtemp_get_text)
{
	char *ptext;
	long pwbo;
	int len;

	// NOTE: I don't quite understand why do I need all these
	// len + 1 and len - 1 stuff below, but it works now

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l", &pwbo) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()				// This is an error, so return NULL

	len = wbGetTextLength((PWBOBJ)pwbo) + 1;
	ptext = emalloc(len + 1);
	if(!ptext)
		RETURN_NULL()				// This is an error, so return NULL
	else {
		wbGetText((PWBOBJ)pwbo, ptext, len);

		if(*ptext) {
			RETURN_STRINGL(ptext, len - 1, TRUE)
		} else
			RETURN_STRING("", TRUE)	// This is a valid empty string
	}
}

/*
ZEND_FUNCTION(wb_get_item_value)
{
    UINT i;
	long item;
    BYTE *items;
	long pwbo;

	item = 0;
    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "ll", &pwbo, &item) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL();

	if(((PWBOBJ)pwbo)->uClass == ListView) {

		items = wbGetListViewCheckedItems((PWBOBJ)pwbo);
		((PWBOBJ)pwbo)->item = item;

		// Build the array

		array_init(return_value);
		for(i = 0; i < strlen(items); i++)
			add_next_index_long(return_value, items[i] == '1');

	} else

		RETURN_LONG(wbGetValue((PWBOBJ)pwbo));
}

ZEND_FUNCTION(wb_set_item_value)
{
    UINT i;
	long item, value;
    BYTE *items;
	long pwbo;

	item = 0;
    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "lll", &pwbo, &item, &value) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL();

	if(((PWBOBJ)pwbo)->uClass == ListView) {

		items = wbGetListViewCheckedItems((PWBOBJ)pwbo);
		((PWBOBJ)pwbo)->item = item;

		// Build the array

		array_init(return_value);
		for(i = 0; i < strlen(items); i++)
			add_next_index_long(return_value, items[i] == '1');

	} else

		RETURN_BOOL(wbSetValue((PWBOBJ)pwbo, value));
}

*/

//------------------------------------------------------------------ END OF FILE
