/*******************************************************************************

 WINBINDER - A native Windows binding

 Copyright  2004 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 Common header for WinBinder: all exported functions and constants

*******************************************************************************/

#ifndef _WB_H
#define _WB_H

//----------------------------------------------------------------- DEPENDENCIES

// Compiler-specific options

#ifndef __LCC__
#	define WIN32_LEAN_AND_MEAN		// For faster compilation
#endif

#ifdef __GNUC__						// For GCC
#	define WINVER 0x501				// Define constants for recent Windows controls
#	define _WIN32_IE WINVER
#endif

#ifdef __POCC__						// For Pelles C
#	ifndef WIN32
#		define WIN32
#	endif
#	define _MAX_PATH				260		// From stdlib.h
#	define _MAX_FNAME				256		// From stdlib.h
#	define __POCC__OLDNAMES					// For access, itoa, stricmp, etc.
#endif

#if defined(_MSC_VER)				// For MSVC (Rick)
#	define __func__ __FUNCTION__
#endif

// Common dependencies

#include "wb_version.h"
#include <windows.h>
#include <commctrl.h>

//-------------------------------------------------------------------- CONSTANTS

// Window types

#define AppWindow	 		100
#define ResizableWindow	 	101
#define PopupWindow	 		102
#define ModalDialog	 		103
#define ToolDialog	 		105
#define ModelessDialog		106

// WinBinder controls

#define Accel				301
#define Calendar			303
#define CheckBox	 		304
#define ComboBox			305
#define EditBox	 			306
#define Frame				307
#define Gauge				308
#define ListView			309
#define HTMLEditBox			311
#define Label				312
#define ListBox				313
#define Menu				314
#define PushButton			316
#define RadioButton			317
#define RTFEditBox			318
#define ScrollBar			319
#define Slider				320
#define Spinner				321
#define StatusBar			323
#define TabControl			324
#define ToolBar				325
#define TreeView			326

// Style flags

#define WBC_VISIBLE				0x00000000
#define WBC_LEFT				0x00000000
#define WBC_ENABLED				0x00000000
#define WBC_INVISIBLE			0x00000001
#define WBC_DISABLED			0x00000002	// Equivalent to WS_DISABLED
#define WBC_IMAGE				0x00000004	// Control has an image
#define WBC_BORDER				0x00000008	// Control has a border
#define WBC_NOTIFY				0x00000010	// lparam contains additional notification messages
#define WBC_RIGHT				0x00000020
#define WBC_READONLY			0x00000040
#define WBC_LINES				0x00000080	// Grid lines, tab ticks
#define WBC_MASKED				0x00000100	// Password in edit fields
#define WBC_COLLAPSED			0x00000200	// Minimize to the taskbar status area
#define WBC_NUMBER				0x00000400	// Numeric edit fields
#define WBC_CHECKBOXES			0x00010000
#define WBC_ELLIPSIS			0x00020000
#define WBC_SORT				0x00040000
#define WBC_CENTER				0x10000000	// Must be a very high value because of wbSetWindowPosition()

// Notification messages

#define WBC_DEFAULT				0x00000000	// Default notification messages
#define WBC_GETFOCUS			0x00000001	// Notification passed as lParam in callback
#define WBC_DBLCLICK			0x00000002	// Notification passed as lParam in callback

// Other WBC_ constants

#define WBC_BEEP				0xFFFFFFFF	// See MessageBeep() doc

// File search flags

#define FSM_FILES				0x00000001	// Files
#define FSM_DIRS				0x00000002	// Directories
#define FSM_INCLUDEPATH			0x00000004	// Include full path for files and dirs
#define FSM_BRACKETS			0x00000008	// Include brackets for dirs
#define FSM_NOEXTENSION			0x00000010	// Do not display file extensions
#define FSM_NOPARENTDIR			0x00000020	// No current or parent directories

// Font attributes

#define FTA_NORMAL				0x00000000
#define FTA_LEFT				0x00000000
#define FTA_TOP					0x00000000
#define FTA_BOLD				0x00000001
#define	FTA_ITALIC				0x00000002
#define FTA_UNDERLINE			0x00000004
#define FTA_STRIKEOUT			0x00000008
#define FTA_SHADOW				0x00000010
#define FTA_HIGHLIGHT			0x00000020
#define FTA_HCENTER				0x00000100
#define FTA_RIGHT				0x00000200
#define FTA_VCENTER				0x00000400
#define FTA_BOTTOM				0x00000800
#define FTA_WORDBREAK   		0x00001000
#define FTA_CALCRECT    		0x00002000
#define FTA_PARSEPREFIX			0x00004000
#define FTA_EMBOSSED			0x00010000

// Other constants

#define MAX_TABS				16		// Maximum number of tabs in a TabControl
#define MAX_TABCONTROLS			512		// Maximum number of controls in a TabControl
#define MAX_LISTVIEWCOLS		256		// Maximum columns in a ListView
#define MAX_ACCELS				256		// Maximum number of keyboard accelerators
#define MAX_ERR_MESSAGE			1024
#define MAX_ITEM_STRING			1024
#define MAX_TREEVIEW_LEVELS		25
#define LVIS_CHECKED			0x2000

// Strings

#define RICHEDITCONTROL			"RichEdit20A"
#define TAB_PAGE_CLASS			"wbTabPage"

// Custom WinBinder messages

#define WBWM_NOTIFYICON			(WM_USER + 2)

//------------------------------------------------------- GENERAL-PURPOSE MACROS

#define BITTEST(a, b)		(((a) & (b)) == (b))
#define NUMITEMS(array)		(sizeof(array) / sizeof(*array))

#ifndef MIN
#define MIN(a,b)			(((a) < (b)) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b)			(((a) > (b)) ? (a) : (b))
#endif

//------------------------------------------------------------------- STRUCTURES

// WinBinder object (window, control, or menu), stored with SetWindowLong() and GWL_USERDATA

typedef struct _wbo {			// wbo
	HANDLE handle;				// Control handle
	UINT id;					// Control identifier
	UINT uClass;				// Object class
	int item;					// Item index
	int subitem;				// Subitem index
	DWORD style;				// WinBinder style
	struct _wbo *parent;		// Parent window
	LPTSTR pszCallBackFn;		// Callback function
	LPARAM lparam;				// Additional information
} WBOBJ, *PWBOBJ;

// Item description

typedef struct {				// wbi
	UINT id;
	UINT index;
	LPCTSTR pszCaption;
	LPCTSTR pszHint;
	LPCTSTR pszImage;
	LPARAM lparam;				// Additional information
} WBITEM, *PWBITEM;

// Tab control data

typedef struct {				// td
	UINT nCtrls;
	UINT nPages;
	struct {
		HWND hwnd;
	} page[MAX_TABS];
	struct {
		HWND hwnd;
		UINT id;
		BYTE nTab;
	} ctrl[MAX_TABCONTROLS];
} TABDATA, *PTABDATA;

// Treeview data

typedef struct {				// trdt
	HTREEITEM hLast;
	HTREEITEM hParent[MAX_TREEVIEW_LEVELS];
	BOOL bCustomImages[MAX_TREEVIEW_LEVELS];
	int nLastLevel;
} TREEDATA, *PTREEDATA;

// Simplified font structure

typedef struct {				// ft
	LPTSTR pszName;
	int nHeight;
	COLORREF color;
	DWORD dwFlags;
} FONT, *PFONT;

//------------------------------------------------- GLOBAL VARIABLE DECLARATIONS

extern HINSTANCE	hAppInstance;		// Instance handle
extern HWND			hCurrentDlg;		// Current dialog box
extern HFONT		hIconFont;			// Icon font
extern WNDPROC		lpfnTabProcOld;		// Original tab control procedure
extern WNDPROC		lpfnLabelProcOld;	// Original label (static) control procedure
extern COLORREF		clrTabs;			// Color for tab control backgrounds
extern HBRUSH		hbrTabs;			// Brush for tab control backgrounds

//------------------------------------------------------------ PUBLIC PROTOTYPES

void		SetStatusBarId(int id);
void		SetToolBarHandle(HWND hCtrl);
BOOL		IsBitmap(HANDLE handle);
BOOL		IsIcon(HANDLE handle);
BOOL		RegisterClasses(void);
BOOL		RegisterControlInTab(PWBOBJ pwboParent, PWBOBJ pwbo, UINT id, UINT nTab);
LRESULT CALLBACK TabProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
LPTSTR		WinPath(LPTSTR pszPath);

// WB_BITMAP.C

BOOL		wbSaveBitmap(HBITMAP hbm, LPCTSTR pszFileName);
DWORD		wbGetBitmapDimensions(HBITMAP hbm);
HBITMAP 	wbReplaceColors(HDC cvTarget, HBITMAP hbm, COLORREF clTransparent, COLORREF clNew);
HBITMAP 	wbCreateBitmap(int nWidth, int nHeight);
HBITMAP 	wbCreateMask(HBITMAP hbm, COLORREF clTransparent);
BOOL		wbDestroyBitmap(HBITMAP hbm);
HDC			wbCreateBitmapDC(HBITMAP hbm);
BOOL		wbCopyBits(HDC hdc, HBITMAP hBitmap, int xTarget, int yTarget);
BOOL		wbCopyPartialBits(HDC hdc, HBITMAP hBitmap, int xTarget, int yTarget, int nWidth, int nHeight, int xSource, int ySource);
BOOL		wbMaskPartialBits(HDC hdc, HBITMAP hbmImage, HBITMAP hbmMask, int xTarget, int yTarget, int nWidth, int nHeight, int xSource, int ySource);
HANDLE		wbLoadImage(LPCTSTR pszImageFile, int nIndex);

// WB_CONTROL.C

PWBOBJ		wbCreateControl(PWBOBJ pwboParent, UINT uWinBinderClass, LPCTSTR pszCaption, int xPos, int yPos, int nWidth, int nHeight, UINT id, DWORD dwWinBinderStyle, long lParam);
PWBOBJ		wbGetControl(PWBOBJ pwboParent, int id);
BOOL		wbCreateItem(PWBOBJ pwbo, LPCTSTR pszItemText);
DWORD		wbGetItemCount(PWBOBJ pwbo);
BOOL		wbSetImage(PWBOBJ pwbo, HANDLE hImage, UINT nIndex, COLORREF clTransp, LPARAM lParam);
BOOL		wbSetFocus(PWBOBJ pwbo);
DWORD		wbGetValue(PWBOBJ pwbo);
BOOL		wbSetValue(PWBOBJ pwbo, DWORD dwValue);
BOOL		wbSetRange(PWBOBJ pwbo, DWORD dwMin, DWORD dwMax);
BOOL		wbGetVisible(PWBOBJ pwbo);
BOOL		wbSetVisible(PWBOBJ pwbo, BOOL bState);
BOOL		wbSetStyle(PWBOBJ pwbo, DWORD dwWBStyle);
BOOL		wbGetEnabled(PWBOBJ pwbo);
BOOL		wbSetEnabled(PWBOBJ pwbo, BOOL bState);
BOOL		wbSetText(PWBOBJ pwbo, LPCTSTR pszText);
BOOL		wbGetText(PWBOBJ pwbo, LPTSTR pszText, UINT nMaxChars);
UINT		wbGetTextLength(PWBOBJ pwbo);
BOOL		wbClearItems(PWBOBJ pwbo);
BOOL		wbRefreshControl(PWBOBJ pwbo, BOOL bNow);
DWORD		wbGetSelected(PWBOBJ pwbo);

// WB_CONTROL_COMBO.C

BOOL		wbCreateComboItem(PWBOBJ pwbol, LPCTSTR pszItem, BOOL bCombo);

// WB_CONTROL_LISTVIEW.C

int			wbCreateListViewItem(PWBOBJ pwbo, int nItem, int nImage, LPCTSTR pszText);
BOOL		wbDeleteListViewItem(PWBOBJ pwbo, int nItem);
BOOL		wbCreateListViewImages(PWBOBJ pwbo, HBITMAP hbmImage, int nImages, COLORREF clTransparent);
BOOL		wbCreateListViewColumn(PWBOBJ pwbo, int nCol, LPTSTR lpszItem, int nWidth);
BOOL		wbSetListViewItemText(PWBOBJ pwbo, int nItem, int nSubItem, LPCTSTR pszText);
BOOL		wbSetListViewItemChecked(PWBOBJ pwbo, int nItem, BOOL bChecked);
int			wbGetListViewItemCount(PWBOBJ pwbo);
BOOL		wbSetListViewItemImage(PWBOBJ pwbo, int nItem, int nSubItem, int nImage);
BOOL		wbGetListViewItemChecked(PWBOBJ pwbo, int nItem);
BOOL		wbGetListViewItemText(PWBOBJ pwbo, int nItem, int nSubItem, LPTSTR pszText, int nChar);
BOOL		wbClearListViewColumns(PWBOBJ pwbo);
BOOL		wbSelectListViewItem(PWBOBJ pwbo, int nItem, BOOL bState);
BOOL		wbSelectAllListViewItems(PWBOBJ pwbo, BOOL bState);
int			wbGetListViewCheckedItems(PWBOBJ pwbo, int *pbItems);
int			wbGetListViewSelectedItems(PWBOBJ pwbo, int *pbItems);
int			wbGetListViewColumns(PWBOBJ pwbo, int *pwidths);
BOOL		wbSetListViewColumnWidth(PWBOBJ pwbo, int nCol, int nWidth);
BOOL		wbSetListViewColumnWidths(PWBOBJ pwbo, int *pwidths);

// WB_CONTROL_MENU.C

PWBOBJ		wbCreateMenu(PWBOBJ pwboParent, PWBITEM pitem[], int nItems);
BOOL		wbGetMenuItemChecked(PWBOBJ pwbo);
BOOL		wbSetMenuItemChecked(PWBOBJ pwbo, BOOL bState);
BOOL		wbSetMenuItemSelected(PWBOBJ pwbo);
BOOL		wbSetMenuItemImage(PWBOBJ pwbo, HANDLE hImage);

// WB_CONTROL_TOOLBAR.C

PWBOBJ		wbCreateToolbar(PWBOBJ pwboParent, PWBITEM pitem[], int nItems, int nBtnWidth, int nBtnHeight, HBITMAP hbm);

// WB_CONTROL_TREEVIEW.C

BOOL		wbCreateTreeViewImages(PWBOBJ pwbo, HBITMAP hbmImage, int nImages, COLORREF clTransparent);
HTREEITEM	wbAddItemToTree(PWBOBJ pwbo, int nLevel, LPTSTR lpszItem, int nImage, int nSelectedImage);
BOOL		wbSetTreeViewSelectedItem(PWBOBJ pwbo, LPTSTR lpszItem, int nImage, int nSelectedImage);

// WB_CONTROL_TAB.C

BOOL		wbSetTabControlText(PWBOBJ pwboTab, LPCTSTR pszText);
BOOL		wbCreateTabItem(PWBOBJ pwbo, LPCTSTR pszItem);
BOOL		wbSelectTab(PWBOBJ pwboTab, int nItem);

// WB_GDI.C

COLORREF	wbGetPixel(HANDLE hImage, int xPos, int yPos);
BOOL		wbSetPixel(HANDLE hImage, int xPos, int yPos, COLORREF color);

// WB_LOWLEVEL.C

LPARAM		wbSendMessage(PWBOBJ pwbo, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL		wbSetWindowStyle(PWBOBJ pwbo, DWORD dwStyle, BOOL bExtended);

// WB_GENERIC.C

BOOL		wbIsWBObj(void *pwbo);
PWBOBJ		wbGetWBObj(HANDLE hwnd);
BOOL		wbGetTextSize(PSIZE psizText, LPCTSTR pszString);

// WB_SYSDLG.C

BOOL		wbSysDlgOpen(PWBOBJ pwboParent, LPCTSTR pszTitle, LPCTSTR pszFilter, LPCTSTR pszPath, LPTSTR pszFileName);
BOOL		wbSysDlgSave(PWBOBJ pwboParent, LPCTSTR pszTitle, LPCTSTR pszFilter, LPCTSTR pszPath, LPTSTR pszFileName);
BOOL		wbSysDlgPath(PWBOBJ pwboParent, LPCTSTR pszTitle, LPCTSTR pszPath, LPTSTR pszSelPath);
COLORREF	wbSysDlgColor(PWBOBJ pwboParent, LPCTSTR pszTitle, COLORREF color);
PFONT		wbSysDlgFont(PWBOBJ pwboParent, LPCTSTR pszTitle, PFONT pfont);

// WB_WINDOW.C

PWBOBJ		wbCreateWindow(PWBOBJ pwboParent, UINT uWinBinderClass, LPCTSTR pszCaption, UINT xPos, UINT yPos, int nWidth, int nHeight, DWORD dwWinBinderStyle, long lParam);
BOOL		wbDestroyWindow(PWBOBJ pwbo);
BOOL		wbSetWindowHandler(PWBOBJ pwbo, LPCTSTR pszHandler);
BOOL		wbSetWindowSize(PWBOBJ pwbo, int nWidth, int nHeight);
BOOL		wbSetWindowPosition(PWBOBJ pwbo, int xPos, int yPos, PWBOBJ pwboParent);
void *		wbGetAppInfo(LPCTSTR pszInfo, BOOL *pbString, LPTSTR pszString, UINT uLen);
BOOL		wbSetAppInfo(LPCTSTR pszInfo, BOOL *pbString, DWORD dwValue);
BOOL		wbSetTimer(PWBOBJ pwbo, int id, UINT uPeriod);
BOOL		wbSortLVColumn(PWBOBJ pwbo, int nSubItem, BOOL bAscending);

// WB_WINSYS.C

BOOL		wbEnd(void);
BOOL		wbInit(void);
BOOL		wbSetAccel(HACCEL hAccTable);
WPARAM		wbMainLoop(void);
BOOL		wbPlaySystemSound(int nStyle);
BOOL		wbShowLastError(LPCTSTR pszCaption, BOOL bMessageBox);
int			wbMessageBox(PWBOBJ pwboParent, LPCTSTR pszText, LPCTSTR pszCaption, UINT nStyle);
BOOL		wbExec(LPCTSTR pszPgm, LPCTSTR pszParm, BOOL bShowWindow);
BOOL		wbFindFile(LPTSTR pszFileName, UINT uLen);
BOOL		wbReadRegistryKey(LPCTSTR pszKey, LPTSTR pszSubKey, LPTSTR pszEntry, LPTSTR pszValue, int *pnBufLen);
BOOL		wbWriteRegistryKey(LPCTSTR pszKey, LPTSTR pszSubKey, LPTSTR pszEntry, LPCTSTR pszValue, DWORD dwValue, BOOL bString);
void *		wbGetSystemInfo(LPCTSTR pszInfo, BOOL *pbString, LPTSTR pszString, UINT uLen);
BOOL		wbSetAccelerators(PWBOBJ pwbo, LPACCEL paccels, int nCount);
DWORD		wbMakeAccelFromString(const char *pszAccel);

// Library-dependent functions

BOOL		wbCallUserFunction(LPCTSTR pszFunctionName, PWBOBJ pwboParent, PWBOBJ pctrl, UINT id, LPARAM lParam);
BOOL		wbError(int nType, LPCTSTR pszFmt, ...);
void		*wbAlloc(size_t size);
void		*wbCalloc(size_t nmemb, size_t size);
void		wbFree(void *ptr);
void		*wbRealloc(void *ptr, size_t size);
char		*wbStrDup(const char *string);
char		*wbStrnDup(const char *string, size_t size);

//------------------------------------------------------------------ END OF FILE

#endif // _WB_H
