/*******************************************************************************

 WINBINDER - A native Windows binding

 Copyright  2004 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 Treeview control

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "wb.h"

//----------------------------------------------------------- EXPORTED FUNCTIONS

/* Create an ImageList for a Treeview */

BOOL wbCreateTreeViewImages(PWBOBJ pwbo, HBITMAP hbmImage, int nImages, COLORREF clTransparent)
{
	HBITMAP hbmMask;
	static HIMAGELIST hi;
	BITMAP bm;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	if(hbmImage && nImages) {
		GetObject(hbmImage, sizeof(BITMAP), (LPSTR) &bm);
		if((bm.bmWidth == 0) | (bm.bmHeight == 0))
			return FALSE;

		if(clTransparent != CLR_INVALID) {
			hbmMask = wbCreateMask(hbmImage, clTransparent);
			hi = ImageList_Create(bm.bmWidth / nImages, bm.bmHeight, ILC_COLORDDB | ILC_MASK, nImages, 0);
			ImageList_Add(hi, hbmImage, hbmMask);
			TreeView_SetImageList(pwbo->handle, hi, TVSIL_NORMAL);
			DeleteObject(hbmMask);
		} else {
			hi = ImageList_Create(bm.bmWidth / nImages, bm.bmHeight, ILC_COLORDDB, nImages, 0);
			ImageList_Add(hi, hbmImage, NULL);
			TreeView_SetImageList(pwbo->handle, hi, TVSIL_NORMAL);
		}
	} else {
		TreeView_SetImageList(pwbo->handle, NULL, TVSIL_NORMAL);
		ImageList_Destroy(hi);
	}
	return TRUE;
}

BOOL wbSetTreeViewSelectedItem(PWBOBJ pwbo, LPTSTR lpszItem, int nImage, int nSelectedImage)
{
	TV_ITEM tvi;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	if((!lpszItem || !*lpszItem))
		return FALSE;

	// Change the current item

	tvi.hItem = TreeView_GetSelection(pwbo->handle);
	if(!tvi.hItem)
		return FALSE;
	if(nImage < 0 || nSelectedImage < 0)
		tvi.mask = TVIF_TEXT;
	else
		tvi.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
	tvi.pszText = lpszItem;
	tvi.cchTextMax = strlen(lpszItem);
	tvi.lParam = 0;
	tvi.iImage = nImage;
	tvi.iSelectedImage = nSelectedImage;

	return TreeView_SetItem(pwbo->handle, &tvi);
}

HTREEITEM wbAddItemToTree(PWBOBJ pwbo, int nLevel, LPTSTR lpszItem, int nImage, int nSelectedImage)
{
	TV_ITEM tvi;
	TV_INSERTSTRUCT tvins;
	PTREEDATA ptrdt	= (PTREEDATA)pwbo->lparam;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	if((!lpszItem || !*lpszItem))
		return NULL;

	// Add the item

	tvi.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM;
	tvi.pszText = lpszItem;
	tvi.cchTextMax = strlen(lpszItem);
	tvi.lParam = 0;
	tvi.iImage = (nImage >= 0 ? nImage : 4);	// Default image indices
	tvi.iSelectedImage = (nImage >= 0 ? nSelectedImage : 5);

	// Where to insert the new item?

	nLevel = MAX(0, nLevel);	// Negative levels don't make sense

	if(nLevel == 0) {							//----------- At root level

		tvins.hParent = TVI_ROOT;
		tvins.hInsertAfter = TVI_LAST;
		ptrdt->hParent[nLevel] = tvins.hParent;

	} else if(nLevel > ptrdt->nLastLevel) {		//----------- As a child item

		nLevel = ptrdt->nLastLevel + 1;	// Can't skip levels
		tvins.hParent = ptrdt->hLast;
		tvins.hInsertAfter = TVI_LAST;
		ptrdt->hParent[nLevel] = tvins.hParent;

		if(tvins.hParent && !ptrdt->bCustomImages[ptrdt->nLastLevel]) {

			TV_ITEM tviParent;

			// Change images of parent item to default "folders": applies them
			// only to parent items that are not flagged as custom

			tviParent.mask = TVIF_HANDLE | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
			tviParent.hItem = tvins.hParent;
			tviParent.iImage = 0;	// Default image indices
			tviParent.iSelectedImage = 1;
			TreeView_SetItem(pwbo->handle, &tviParent);
		}

	} else if(nLevel == ptrdt->nLastLevel) {	//----------- As a sibling

		tvins.hParent = ptrdt->hParent[nLevel];
		tvins.hInsertAfter = TVI_LAST;
		ptrdt->hParent[nLevel] = tvins.hParent;

	} else {									//----------- Back up 1 or more levels

		tvins.hParent = ptrdt->hParent[(ptrdt->nLastLevel > nLevel)];
		tvins.hInsertAfter = TVI_LAST;
		ptrdt->hParent[nLevel] = tvins.hParent;

	}

	tvins.item = tvi;
	ptrdt->nLastLevel = nLevel;
	ptrdt->bCustomImages[nLevel] = (nImage >= 0);	// Flags custom images

	// Insert the item

	ptrdt->hLast = TreeView_InsertItem(pwbo->handle, &tvins);
	return ptrdt->hLast;
}

//------------------------------------------------------------------ END OF FILE
