/*******************************************************************************

 WINBINDER - A native Windows binding

 Copyright  2004 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 Drawing functions

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "wb.h"

//-------------------------------------------------------------------- CONSTANTS

//----------------------------------------------------------------------- MACROS

//----------------------------------------------------------- PRIVATE PROTOTYPES

static HDC GetBitmapHDC(HBITMAP hbm);

//-------------------------------------------------------------------- VARIABLES

//----------------------------------------------------------- EXPORTED FUNCTIONS

COLORREF wbGetPixel(HANDLE handle, int xPos, int yPos)
{
	COLORREF clResult;

	if(!handle)
		return CLR_INVALID;

	if(IsWindow((HWND)handle)) {						// Is the handle a window?

		HDC hdc;

		hdc = GetDC((HWND)handle);
		clResult = GetPixel(hdc, xPos, yPos);
		ReleaseDC((HWND)handle, hdc);
		return clResult;

	} else {

		clResult = GetPixel((HDC)handle, xPos, yPos);	// Is it a DC?
		if(clResult == CLR_INVALID) {					// No, maybe a bitmap
			HDC hdc;

			hdc = GetBitmapHDC((HBITMAP)handle);
			if(hdc) {
				clResult = GetPixel(hdc, xPos, yPos);	// Yes, it's a bitmap
				DeleteDC(hdc);
				return clResult;
			} else {
				return CLR_INVALID;
			}
		} else
			return clResult;
	}
}

BOOL wbSetPixel(HANDLE handle, int xPos, int yPos, COLORREF color)
{
    BOOL bResult;

	if(!handle)
		return CLR_INVALID;

	if(IsWindow((HWND)handle)) {						// Is the handle a window?

		HDC hdc;

		hdc = GetDC((HWND)handle);
		bResult = (SetPixel(hdc, xPos, yPos, color) != (COLORREF)-1);
		ReleaseDC((HWND)handle, hdc);
		return bResult;

	} else {

		bResult = (SetPixel((HDC)handle, xPos, yPos, color) != (COLORREF)-1);		// DC?
		if(!bResult) {									// No, maybe a bitmap
			HDC hdc;

			hdc = GetBitmapHDC((HBITMAP)handle);
			if(hdc) {
				bResult = (SetPixel(hdc, xPos, yPos, color) != (COLORREF)-1);
				DeleteDC(hdc);
				return bResult;
			} else {
				return CLR_INVALID;
			}
		} else
			return bResult;
	}
}

//------------------------------------------------------------ PRIVATE FUNCTIONS

/* Return a DC for the bitmap pbm. Must be followed by a call to DeleteDC. */

static HDC GetBitmapHDC(HBITMAP hbm)
{
	HDC cv, cvBmp;

	if(!hbm)
		return NULL;
	cv = GetDC(NULL);
	if(!cv)
		return NULL;
	cvBmp = CreateCompatibleDC(cv);
	if(!cvBmp) {
		ReleaseDC(NULL, cv);
		return NULL;
	}
	SelectObject(cvBmp, hbm);
	ReleaseDC(NULL, cv);
	return cvBmp;
}

//------------------------------------------------------------------ END OF FILE
